/*	NETMON hostpair.c  (MITRE)	12/18/88	*/

/*
 * hostpair.c:  routines for the host-pair records used to speed
 * logging and do selective and summary logging.
 *
 * By:  A. Mankin
 *
 * Copyright (c) 1989, the MITRE Corporation.  All rights reserved.
 * This software was developed by the MITRE Corporation, McLean VA.
 * It is distributed free of charge with the following conditions:
 *
 *      1.  Redistribution and use in source and binary forms are permitted
 *	    provided that this copyright notice be duplicated in all such
 *	    forms.
 *
 *      2.  MITRE will be credited should the software be used in a
 *          product or written about in any publications.  However,
 *          MITRE will not be referenced as the source in advertisements.
 *
 *      3.  MITRE assumes no legal responsibility for source code and
 *          its subsequent use.  No warranty is expressed or implied.
 *
 *      4.  If any bugs or problems are found, then they should be
 *          reported back to MITRE.
 */

#include "iptrace.h"

/*
 * Include files specific to IP packets.
 */
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/tcp.h>
#define NETMON_KERNEL
#include <net/netmon.h>

int Nflag = 1;

struct addr_rec *arhead = NULL;


#define INADDR_SZ 	(sizeof(struct in_addr))

oldhostpair(ar) 
     struct addr_rec *ar;
{
  if(ar->new)
    return(0);
  else
    return(1);
}

newhostpair(ar)
     struct addr_rec *ar;
{
  if (ar->new) --ar->new;
  return(ar->new);
}


struct addr_rec *get_ip_rec(ipheader)
     struct ip *ipheader;
{			 
     struct hostent *hostent;
     struct addr_rec *ar = arhead, *armatch = NULL;
     int i;

     u_char cataddr[2*INADDR_SZ];

     bcopy((u_char *)&ipheader->ip_src, cataddr, sizeof(cataddr));

     while(ar) {

	  if ( bcmp(ar->cataddr12, cataddr, 2*INADDR_SZ) == 0) {
	       ar->first_seen_direction++;
	       ar->first_dir_summ++;
	       armatch = ar;
	       break;
	  }

	  if ( bcmp(ar->cataddr21, cataddr, 2*INADDR_SZ) == 0) {
	       ar->other_direction++;
	       ar->other_dir_summ++;
	       ar->reversematch++;
	       armatch = ar;
	       break;
	  }
	  ar = ar->next;
	  continue;
     }
     if(armatch)
	  return(armatch);

     ar = (struct addr_rec *)malloc( sizeof(struct addr_rec));
     bzero((char *)ar, sizeof(struct addr_rec));
     ar->next = arhead;
     arhead = ar;
     ar->first_seen_direction++;
     ar->first_dir_summ++;
     
     ar->name1 = malloc ( MAX_NAME_SZ);
     ar->name2 = malloc ( MAX_NAME_SZ);
     ar->cataddr12 = (u_char *)malloc (2*INADDR_SZ);
     ar->cataddr21 = (u_char *)malloc (2*INADDR_SZ);
     
     if (Nflag) {
	  strcpy(ar->name1, inet_ntoa(ipheader->ip_src));
	  strcpy(ar->name2, inet_ntoa(ipheader->ip_dst));
	  goto nolookup;
     }

     hostent = gethostbyaddr((char *)&ipheader->ip_src,
			     INADDR_SZ, AF_INET);
     if (hostent == NULL)
	  strcpy(ar->name1, inet_ntoa(ipheader->ip_src));
     else
	  strcpy(ar->name1, hostent->h_name);
     
     
     hostent = gethostbyaddr((char *)&ipheader->ip_dst,
			     INADDR_SZ, AF_INET);
     if (hostent == NULL)
	  strcpy(ar->name1, inet_ntoa(ipheader->ip_dst));
     else
	  strcpy(ar->name2, hostent->h_name);

   nolookup:
     
     bcopy((u_char *)&ipheader->ip_src, ar->cataddr12, INADDR_SZ);
     
     bcopy((u_char *)&ipheader->ip_dst, ar->cataddr12 + INADDR_SZ, 
	   INADDR_SZ);
     
     bcopy((u_char *)&ipheader->ip_dst, ar->cataddr21, INADDR_SZ);
     
     bcopy((u_char *)&ipheader->ip_src, ar->cataddr21 + INADDR_SZ, 
	   INADDR_SZ);
     
     ar->new = ADDR_REC_MCOUNT;
     return (ar);
     
}


host_matrix(file) 
     FILE *file;
{
     char datestamp[26];
     struct addr_rec *ar, *tempnext; 
     static struct addr_rec *artail; 
     struct timeval ts;
     struct tm *tm;
     int i;

     makedatestamp(datestamp);
     fprintf(file,"\n\t----HTM %s----\n", datestamp);
     
     artail = ar = arhead;

     while(ar) {

          tempnext = ar->next;

	  /*
	   * If there's no activity recorded.
	   */
	  if((!ar->first_seen_direction && !ar->other_direction)||
	     (!ar->first_dir_summ && !ar->other_dir_summ)) {

	    /*
	     * If the record was not used for some number of
	     * loggings, free its space (to keep
	     * down size of table on long runs--N.B. the table
	     * should be hashed with Van's Patricia algorithm.
	     * If it shouldn't be freed, link it to the new
	     * version of the list.  The static variable
	     * artail is used to keep track of the tail.
	     */
	       if(ar->idle++ > ADDR_REC_IDLE_DROP) {

			if(ar == arhead)
				arhead = ar->next;

			free((char *)ar->name1);
			free((char *)ar->name2);
			free((char *)ar->cataddr12);
			free((char *)ar->cataddr21);
			free((char *)ar);
		}
		else {
			artail->next = ar;
			artail = ar;
                }

	  }
	  else  {
	    if (file == HTM) {
	      fprintf(file, "\t%s > %s : %d\n", 
		    ar->name1, ar->name2, ar->first_seen_direction);
	    
	      fprintf(file, "\t%s > %s : %d\n\n", 
		    ar->name2, ar->name1, ar->other_direction);
	    
	      ar->first_seen_direction = 0;
	      ar->other_direction = 0;
	    }
	    if (file == SAMPLE) {
	      fprintf(file, "\t%s > %s : %d\n", 
		    ar->name1, ar->name2, ar->first_dir_summ);
	    
	      fprintf(file, "\t%s > %s : %d\n\n", 
		    ar->name2, ar->name1, ar->other_dir_summ);

	      ar->first_dir_summ = 0;
	      ar->other_dir_summ = 0;
	    }
	  }

	  ar = tempnext;
     }

     fflush(file);

}
