
/*	NETMON iptrace.c  (MITRE)	12/18/88	*/

/*
 * iptrace.c:  Main loop for iptrace and IP-protocol 
 * specific routines.
 *
 * By:  A. Mankin
 *
 * Copyright (c) 1989, the MITRE Corporation.  All rights reserved.
 * This software was developed by the MITRE Corporation, McLean VA.
 * It is distributed free of charge with the following conditions:
 *
 *      1.  Redistribution and use in source and binary forms are permitted
 *	    provided that this copyright notice be duplicated in all such
 *	    forms.
 *
 *      2.  MITRE will be credited should the software be used in a
 *          product or written about in any publications.  However,
 *          MITRE will not be referenced as the source in advertisements.
 *
 *      3.  MITRE assumes no legal responsibility for source code and
 *          its subsequent use.  No warranty is expressed or implied.
 *
 *      4.  If any bugs or problems are found, then they should be
 *          reported back to MITRE.
 */

#include "iptrace.h"

/*
 * Include files specific to IP logging.
 */
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/tcp.h>
#define NETMON_KERNEL
#include <net/netmon.h>

int Tflag = 0;


/*
 * Initialized strings.
 */
char *tcpname = "TCP", 
     *udpname = "UDP", 
     *icmpname = "ICMP", 
     *otherprotoname = "proto";

/*
 * Defines.
 */
#define MINULPBYTES 8

main ()
{

  struct sigvec vec, ovec;
  doconfig(0);
  initNetmon();

  /*
   * Set up to reconfigure on SIGHUP
   */
  vec.sv_handler = config;
  sigvec(SIGHUP, &vec, &ovec);

  /*
   * Set up to dump out a summary on SIGINT.
   */
  vec.sv_handler = dosumm;
  sigvec(SIGINT, &vec, &ovec);


  /*
   * Set up abort on SIGTERM.
   */
  vec.sv_handler = cleanup;
  vec.sv_mask = sigmask(SIGINT);
  sigvec(SIGTERM, &vec, &ovec);

  for (;;) {
    getNetmonRec();
    if (needconfig)
      doconfig(SIGUSR1);
    needconfig = 0;
  }

} 

	 
void iptrace(phdr, cplen, protobuf, ar, noswap, addbackhdrlen)
     char *phdr;
     int cplen, noswap, addbackhdrlen;
     char *protobuf;
     struct addr_rec *ar;
{
     struct ip *ipheader = (struct ip *)phdr;
     int ipwithopts, hasnoulp, hasports, icmphasorig;
     u_short sport, dport;
     struct icmp *icmpheader;
     int type, code;
     char *ulp, *protoname, *bp;
     u_long seqno;
	  

     if (cplen > sizeof(struct ip)) {


	  sport = dport = hasnoulp = hasports = icmphasorig = 0;
	  protoname = NULL;
	  seqno = 0;

	  ipwithopts = (ipheader->ip_hl<<2);

	  if (cplen < ipwithopts + MINULPBYTES) 
	       hasnoulp++;

	  ulp = (char *)ipheader + ipwithopts;

	  switch (ipheader->ip_p) {

	     case IPPROTO_TCP:
	       protoname = tcpname;
	       if(hasnoulp) break;
               if(Tflag)
		  seqno = *((u_long *)ulp + 1);
	       /* Fall through */

	     case IPPROTO_UDP:
	       if(protoname == NULL)
		    protoname = udpname;
	       if(hasnoulp) break;
	       sport = *((u_short *)ulp);
	       dport = *((u_short *)ulp + 1);
	       hasports++;
	       break;

	     case IPPROTO_ICMP:
	       protoname = icmpname;
	       if(hasnoulp) break;
	       icmpheader = (struct icmp *)ulp;
	       type = (int)icmpheader->icmp_type;
	       code = (int)icmpheader->icmp_code;
	       if (((type == ICMP_UNREACH) || 
		   (type == ICMP_SOURCEQUENCH) || 
		   (type == ICMP_REDIRECT) || 
		   (type == ICMP_TIMXCEED)) && 
		   (cplen >= ipwithopts + ICMP_ADVLENMIN))
		    icmphasorig++;
	       break;

	     default:
	       protoname = otherprotoname;
	       break;

	  }

	  if (!noswap)
	    ipheader->ip_len = ntohs(ipheader->ip_len);
	  if (addbackhdrlen)
	    ipheader->ip_len += sizeof(struct ip);
  	  bp = protobuf;
	  sprintf(bp, "iplen %d %s(%d)", ipheader->ip_len,
		  protoname, ipheader->ip_p);
	 
  	  bp = (char *)((int)protobuf + strlen(protobuf));
	  if (ar->reversematch) {
	       ar->reversematch = 0;
	       if(hasports)
		    sprintf(bp," %s.%d > %s.%d", 
			    ar->name2, ntohs(sport), 
			    ar->name1, ntohs(dport));
	       else
		    sprintf(bp," %s > %s", ar->name2, ar->name1);
	  }
	  else {
	       if(hasports)
		    sprintf(bp," %s.%d > %s.%d", 
			    ar->name1, ntohs(sport), 
			    ar->name2, ntohs(dport));
	       else
		    sprintf(bp," %s > %s", ar->name1, ar->name2);
	  }
	  
  	  bp = (char *)((int)protobuf + strlen(protobuf));
	  if(ipheader->ip_p == IPPROTO_ICMP) {
	       sprintf(bp, " type %d code %d ", type, code);

	       if(type == ICMP_SOURCEQUENCH) {
		    bp = (char *)((int)protobuf + strlen(protobuf));
		    sprintf(bp, "QUENCH ");
	       }
	       else if(type == ICMP_REDIRECT) {
		    bp = (char *)((int)protobuf + strlen(protobuf));
		    sprintf(bp, "REDIRECT to %s ",
			    inet_ntoa((u_long)(icmpheader->icmp_void)));
	       }

	       if(icmphasorig) {
		 char *icmpiphdr = (char *)&icmpheader->icmp_ip;
		 struct addr_rec *icmpar = get_ip_rec(icmpiphdr);

		 bp = (char *)((int)protobuf + strlen(protobuf));
		 iptrace(icmpiphdr, 
			 (cplen - ipwithopts - ICMP_MINLEN), 
			 bp, icmpar, 0);
	       }
	       
	  }

  	  bp = (char *)((int)protobuf + strlen(protobuf));
          sprintf(bp, " %ld", ntohl(seqno));

  	  bp = (char *)((int)protobuf + strlen(protobuf) + 1);
	  *bp = 0;
		    
	}
}

