/* 	NETMON iptrace.h   (MITRE)        12/18/88	*/

/*
 * iptrace.h:  Compile-time defines, common externs for iptrace.
 *
 * By: A. Mankin
 *
 * Copyright (c) 1989, the MITRE Corporation.  All rights reserved.
 * This software was developed by the MITRE Corporation, McLean VA.
 * It is distributed free of charge with the following conditions:
 *
 *      1.  Redistribution and use in source and binary forms are permitted
 *	    provided that this copyright notice be duplicated in all such
 *	    forms.
 *
 *      2.  MITRE will be credited should the software be used in a
 *          product or written about in any publications.  However,
 *          MITRE will not be referenced as the source in advertisements.
 *
 *      3.  MITRE assumes no legal responsibility for source code and
 *          its subsequent use.  No warranty is expressed or implied.
 *
 *      4.  If any bugs or problems are found, then they should be
 *          reported back to MITRE.
 */

#include <sys/file.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/time.h>
#include <sys/socket.h>

#include <string.h>
#include <stdio.h>
#include <netdb.h>
#include <errno.h>

/*
 * Configuration file and other compile time params.
 */
#define IPTRACE_CONF_FILE "/usr/local/etc/iptrace.conf"
/*
 * Verbose mode with output to stderr.
 */
#define VERBOSE 1
/*
 * Constants for configuration file parsing.
 */
#define MAX_CONFIG_TOKENS 3
#define CONFIG_TOKEN_SIZE 40
/* 
 * 1/2 of maximum bytes for log.
 */
#define NETMONLOG 512		
/*
 * Minutes between entries if HTM feature is
 * selected but htminterval is not configured.
 */
#define DEFAULT_HTMINTERVAL 60 	

/*
 * External definitions.
 */
extern FILE *TRACE, *HTM, *SAMPLE;
extern int verbose;
extern int needconfig;
extern int errno;
extern int Nflag;
extern int Xflag;
extern int Tflag;

/*
 * Function type declarations.
 */
struct tm *localtime();
struct hostent *gethostbyaddr();
FILE *fopen();
char *malloc();
char *strcpy();
char *inet_ntoa();
char *kif_name();

/*
 * Signal handlers.
 */
int config();
int dotimer();
int cleanup(); 
int dosumm(); 

/*
 * Address record structure used for speeding up 
 * and controlling logging.
 */
struct addr_rec {
     struct addr_rec *next;
     struct addr_rec *prev;
     u_char *cataddr12;
     u_char *cataddr21;
     char *name1;
     char *name2;
#define MAX_NAME_SZ	32
     int first_seen_direction;
     int first_dir_summ;
     int other_direction;
     int other_dir_summ;
     int reversematch;
     short idle;
#define ADDR_REC_IDLE_DROP    2000
     short new;
#define ADDR_REC_MCOUNT       20
};

struct addr_rec *get_ip_rec();








