/* 	NETMON netkmem.c  (MITRE)        12/18/88		*/

/*
 * netkmem.c:  kmem routines for net tools in NETMON.
 *
 * By: A. Mankin
 *
 * Copyright (c) 1989, the MITRE Corporation.  All rights reserved.
 * This software was developed by the MITRE Corporation, McLean VA.
 * It is distributed free of charge with the following conditions:
 *
 *      1.  Redistribution and use in source and binary forms are permitted
 *	    provided that this copyright notice be duplicated in all such
 *	    forms.
 *
 *      2.  MITRE will be credited should the software be used in a
 *          product or written about in any publications.  However,
 *          MITRE will not be referenced as the source in advertisements.
 *
 *      3.  MITRE assumes no legal responsibility for source code and
 *          its subsequent use.  No warranty is expressed or implied.
 *
 *      4.  If any bugs or problems are found, then they should be
 *          reported back to MITRE.
 */

/*
 * Include files.
 */
#include "iptrace.h"

#include <net/if.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>

#include <nlist.h>

/*
 * Defines.
 */
#define MAXNIF  4

/*
 * Structure declarations.
 */
struct kif_tab {
    char name[16];
    caddr_t hook;
};

struct kif_tab kiftab[MAXNIF];

/*
 * Namelist declaration for interface structure
 * routines in this file.
 */

struct nlist nl[] = {
  {"_ifnet"},
#define NL_IFNET    0
  {0}
};

/*
 * External declaration.
 */
extern errno;

/*
 * Global variables.
 */
int n_interfaces;
int kmem;

/*
 * Get names/addresses of interfaces.
 */
kif_init()
{
	struct ifnet *ifnetaddr, ifnet;
	struct kif_tab *ift = kiftab;

        init_for_kmem(nl);

	ifnetaddr = (struct ifnet *)nl[NL_IFNET].n_value;

	if (ifnetaddr == 0) {
		printf("ifnet: symbol not defined\n");
		return;
	}
	lseek (kmem, (off_t)ifnetaddr, 0);
	read (kmem, &ifnetaddr, sizeof ifnetaddr);
	while (ifnetaddr) {
		register char *cp;
		char *index ();

		lseek (kmem, (off_t)ifnetaddr, 0);
		read (kmem, &ifnet, sizeof ifnet);
		lseek (kmem, (off_t)ifnet.if_name, 0);
		read (kmem, ift->name, 16);
		ift->name[15] = '\0';
		ift->hook = (caddr_t)ifnetaddr;
		ifnetaddr = ifnet.if_next;
		cp = index (ift->name, '\0');
		*cp++ = ifnet.if_unit + '0';
		*cp = '\0';
		n_interfaces++;
		ift++;
	}

	return;
}

char *kif_name(addr) 
     caddr_t addr;
{
     int k;
     struct kif_tab *ift;

     for (k = 0, ift = kiftab; k < n_interfaces; k++, ift++)
	  if (ift->hook == addr)
	       return(ift->name);
     return(NULL);
}
	  

init_for_kmem(nl) 
     struct nlist *nl;
{
    if ((kmem = open ("/dev/kmem", 0)) < 0) {
      if (verbose)
        fprintf(stderr, "init_for_kmem: open %d\n", errno);
      exit(-1);
    }

    nlist ("/vmunix",nl);
    if (nl[0].n_type == 0) {
        fprintf(stderr, "init_for_kmem: no namelist\n");
	exit(-1);
    }
    return;
}

klookup(off, buf, size)
     caddr_t off;
     caddr_t buf;
     int size;
{
  lseek (kmem, off, 0);
  read (kmem, buf, size);
}


