/*	NETMON netmonbuf.c  (MITRE)	    12/18/88	*/

/*
 * netmonbuf.c:  routines dealing with NETMON and with logging
 * its protocol-independent information.
 *
 * By:  A. Mankin	         
 *
 * Copyright (c) 1989, the MITRE Corporation.  All rights reserved.
 * This software was developed by the MITRE Corporation, McLean VA.
 * It is distributed free of charge with the following conditions:
 *
 *      1.  Redistribution and use in source and binary forms are permitted
 *	    provided that this copyright notice be duplicated in all such
 *	    forms.
 *
 *      2.  MITRE will be credited should the software be used in a
 *          product or written about in any publications.  However,
 *          MITRE will not be referenced as the source in advertisements.
 *
 *      3.  MITRE assumes no legal responsibility for source code and
 *          its subsequent use.  No warranty is expressed or implied.
 *
 *      4.  If any bugs or problems are found, then they should be
 *          reported back to MITRE.
 */

#include "iptrace.h"

/*
 * Include files specific to netmonbuf.
 */
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/inode.h>


#define NETMON_KERNEL
#include <net/netmon.h>

/*
 * Device name for access to kernel NETMON.
 */
char *dev = "/dev/netmon";

/*
 * Buffer for copy from kernel NETMON.
 */
struct netmonstr netbuf[NM_MAXTRACED>>1];
maxbufsize = sizeof(netbuf);


/*
 * Tables of NETMON location and code strings.
 */
#define NM_MCDS 7
char *ipIncodes[NM_MCDS] = {"ipIn", "Forw", "Deliver", "HdrErr", 
"Discard", "Fragment"};
char *ipOutcodes[NM_MCDS] = {"ipOut","Forw", "Request", "NoRoute",
			   "FragFail", "Fragment", "OutError"};
char *ethIncodes[NM_MCDS] = {"ethIn", "Nucast", "Discard"};
char *ethOutcodes[NM_MCDS] = {"ethOut", "Nucast", "Discard", "atSendReqd",
			   "atReqFail", "Deq"};
char *x25Incodes[NM_MCDS] = {"x25In", "Discard"};
char *x25Outcodes[NM_MCDS] = {"x25Out", "Discard", "NewCircReqd",
			    "NewCircFail", "Deq"};
char *routecodes[NM_MCDS] = {"route","Netdown"};
char *ifOutcodes[NM_MCDS] = {"ifOut","Discard", "Netdown", "OutError"};

char **netcode;

/*
 * Global variables.
 */
int fd;
int Xflag = 0;

initNetmon()
{
  if ((fd = open(dev, O_RDONLY, IREAD)) < 0) {
    if (verbose)
      fprintf(stderr,"initNetmon: open: %d", errno);
    exit(-1);
  }
  
  kif_init();

}

getNetmonRec()
{
     int res, i, recsread;
     register struct netmonstr *nt;
     static u_short saveseqno = 0;
     int numdropped;
     /*
      * Buffers for logging each record.
      */
     char nmbuf[NETMONLOG], protobuf[NETMONLOG], *bp;
     struct tm *tm;
     int badloc;
     int noswap, addbackhdrlen, infoifname;
     int j;
     struct addr_rec *ar;
     int sampleunprinted = 0;


     res = read(fd, netbuf, maxbufsize);

     if (res <= 0) {
       if(verbose)
	  fprintf(stderr, "getNetmonRec: read: %d", errno);
       exit(-1);
     }
     
     /*
      * For each record in the NETMON buffer:
      *
      * if it has a protocol header, do proto-specific address 
      * lookup and (unless limiting logging) protocol formatting.
      *	  
      * do protocol independent formatting and log the record.
      *
      */
      recsread = res/(sizeof(struct netmonstr));
      for (i = 0, nt = netbuf; i < recsread; i++, nt++) 
      {
	  badloc = 0;

	 /*
	  * Check for drops by NETMON.
	  */
	  if(i == recsread - 1)
	    saveseqno = nt->seqno;

	  if(saveseqno && (i == 0) && (nt->seqno != saveseqno + 1)) {

	    numdropped = (int)(nt->seqno - saveseqno);

	    if(TRACE)
	      fprintf(TRACE, "DROPPED %d NETMON RECORDS\n", numdropped);
	    if(SAMPLE)
	      fprintf(SAMPLE, "DROPPED %d NETMON RECORDS\n", numdropped);
	    if(HTM)
	      fprintf(HTM, "DROPPED %d NETMON RECORDS\n", numdropped);
	    if(verbose)
	      fprintf(stderr, "DROPPED %d NETMON RECORDS\n", numdropped);
	  }       

         /*
	  * Protocol-specific stuff.
	  */

	  noswap = infoifname = addbackhdrlen = 0;

	  switch(nt->loc) {
	     case IP_IN:
	       netcode = ipIncodes;
	       noswap++;	
	       infoifname++;
	       if (nt->code == IP_IN_DELIVER)
		 addbackhdrlen++;
	       break;
	     case IP_OUT:
	       netcode = ipOutcodes;
	       infoifname++;
	       break;
	     case ETH_IN:
	       netcode = ethIncodes;
	       break;
	     case ETH_OUT:
	       netcode = ethOutcodes;
	       break;
	     case X25_IN:
	       netcode = x25Incodes;
	       break;
	     case X25_OUT:
	       netcode = x25Outcodes;
	       break;
	     case ROUTE:
	       netcode = routecodes;
	       infoifname++;
	       break;
	     case IF_OUT:
	       netcode = ifOutcodes;
	       infoifname++;
	       break;
	     default:
	       badloc = 1;
	       break;
	  };

	  if (badloc || ffs(nt->code) > NM_MCDS) {
	       if (verbose)
		 fprintf(stderr, "%ld %s:%d %x\n", nt->seqno,
		       (badloc?"Unknown Loc":"Bad Code"),
		       nt->loc, nt->code);
	       badloc = 0;
	       continue;
	  }

          if (nt->len) {	
	    if (Xflag) {
	      bp = protobuf;
	      for(j=0; j < nt->len; j++, bp += 3)
		sprintf(protobuf, " %02x", nt->data[j]);
	    }
	    else {
	      ar = get_ip_rec(nt->data);

	      if (SAMPLE && oldhostpair(ar)) {
		sampleunprinted++;
		if (!TRACE)
		  continue;
	      }

	      iptrace(nt->data, nt->len, protobuf, ar, 
		      noswap, addbackhdrlen);
	    }
	  }
	  else if (SAMPLE && !TRACE)
	    continue;

	  /*
	   * Protocol-independent stuff.
	   */
	  bp = nmbuf;

	  tm = localtime(&nt->ts.tv_sec);
	  
	  if (nt->ts.tv_usec >= 1000000) {
	       tm->tm_sec++;
	       nt->ts.tv_usec = 0;
	  }

	  sprintf(nmbuf, "%X %02d:%02d:%02d.%02d %s%s>%d ",
		  nt->id,
		  tm->tm_hour, tm->tm_min, tm->tm_sec,
		  nt->ts.tv_usec/10000,
		  netcode[0],
		  ((j = ffs(nt->code)) ? netcode[j]: ""),
		  (char)nt->ql);


	  bp = (char *)((int)nmbuf + strlen(nmbuf));

	  if(nt->info && infoifname)
	       sprintf(bp, "%s ", kif_name(nt->info));
	  else
	       sprintf(bp, "%d ", nt->info);
	  
	  bp = (char *)((int)nmbuf + strlen(nmbuf) +1);
	  *bp = 0;

	  if (nt->len) {
	    if (TRACE)
	      fprintf(TRACE,"%s%s\n", nmbuf, protobuf);
	    if (SAMPLE && (newhostpair(ar))) {
	      if (sampleunprinted) {
		fprintf(SAMPLE, "Not recording %d packets\n",
			sampleunprinted);
		sampleunprinted = 0;
	      }
	      fprintf(SAMPLE,"%s%s\n", nmbuf, protobuf);
	    }
	  }
	  else if (TRACE)
	      fprintf(TRACE,"%s\n", nmbuf);
	  
     }

     if (TRACE)
       fflush(TRACE);
     if(SAMPLE)
       fflush(SAMPLE);
     return;
}
