/*
 * netmon.h:  UNIX kernel network protocol logging.
 *
 * By: A. Mankin
 *
 * Copyright (c) 1989, the MITRE Corporation.  All rights reserved.
 * This software was developed by the MITRE Corporation, McLean VA.
 * It is distributed free of charge with the following conditions:
 *
 *      1.  Redistribution and use in source and binary forms are permitted
 *	    provided that this copyright notice be duplicated in all such
 *	    forms.
 *
 *      2.  MITRE will be credited should the software be used in a
 *          product or written about in any publications.  However,
 *          MITRE will not be referenced as the source in advertisements.
 *
 *      3.  MITRE assumes no legal responsibility for source code and
 *          its subsequent use.  No warranty is expressed or implied.
 *
 *      4.  If any bugs or problems are found, then they should be
 *          reported back to MITRE.
 */


#define NM_MAXTRACED	400		/* Space allowed for tracing */
#define NM_MAXDATA	60	

#define NETMON_ALLDRIVERS  		/* Probes in all drivers-undefine if reports
					 * by IP of error returns from network interface
					 * are needed.
					 */


#define NM_NOCOPY	0		/* Associated record */
#define NM_COPYNOW	1		/* Main record (copy headers) */

#define NM_NOQ		(-1)		

/*
 * Locations in protocols.
 */
#define IP_IN		1
#define IP_OUT		2
#define ETH_IN		3
#define ETH_OUT		4
#define X25_IN	 	5
#define X25_OUT 	6	
/*
 * Location in routing routines.
 */
#define ROUTE		7
/*
 * Location at the error-returning interface
 * of network drivers.
 */
#define IF_OUT		8

/*
 * Codes
 */
#define NM_NOCODE		0	/* Normal code--don't reuse */

#define IP_IN_FORW		0x1
#define IP_IN_DELIVER		0x2
#define IP_IN_HDRERR		0x4
#define IP_IN_DISCARD		0x8
#define IP_IN_FRAGMENT		0x10

#define IP_OUT_FORW		0x1
#define IP_OUT_REQUEST		0x2
#define IP_OUT_NOROUTE		0x4
#define IP_OUT_FRAGFAIL		0x8
#define IP_OUT_FRAGMENT		0x10
#define IP_OUT_ERROR		0x20

#define ROUTE_NETDOWN		0x1

#define IF_OUT_DISCARD		0x1
#define IF_OUT_NETDOWN		0x2
#define IF_OUT_ERROR		0x4


#define ETH_IN_NUCAST		0x1
#define ETH_IN_DISCARD		0x2

#define ETH_OUT_NUCAST		0x1
#define ETH_OUT_DISCARD		0x2
#define ETH_OUT_ATSENDREQD	0x4
#define ETH_OUT_ATREQFAIL	0x8
#define ETH_OUT_DEQ	        0x10

#define X25_IN_DISCARD		0x1

#define X25_OUT_DISCARD		0x1
#define X25_OUT_NEWCIRCREQD	0x2
#define X25_OUT_NEWCIRCFAIL	0x4
#define X25_OUT_DEQ	        0x8

extern netmonp;	/* Turns on packet/queue tracing. */

#ifdef NETMON_KERNEL

struct netmonstr {
     u_short seqno;	/* Sequence no. for stats on overflow. */
     short len;		/* Bytes of protocol header copied. */
     caddr_t id;	/* Mbuf addr, to match mult. records of 1 pkt. */
     struct timeval ts;	/* Timestamp. */
     u_short loc;	/* Location of probe. */
     u_short code;	/* Event code. */
     short ql;		/* Queue length (minus current pkt). */
     short family;	/* Protocol family (e.g. AF_INET). */
     u_long info;	/* Location-dependent extra info: 
			 * x25In, x25Out   LCN
			 * ipIn, ipOut     interface id
			 */
			
     u_char data[NM_MAXDATA];
};

struct netmonstat {			/* Stats about netmon itself */
    int records;
    int readbufferflips;
    int fullbufferflips;
    int fullbufferdrops;
    int readwakes;
    int readsleeps;
} netmonstat;

#endif
